/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */

#ifndef INCLUDED_XMLOFF_XMLPRCON_HXX
#define INCLUDED_XMLOFF_XMLPRCON_HXX

#include <xmloff/xmlictxt.hxx>
#include <xmloff/xmlimppr.hxx>

#include <vector>

struct XMLPropertyState;

class XMLOFF_DLLPUBLIC SvXMLPropertySetContext : public SvXMLImportContext
{
protected:
    sal_Int32 mnStartIdx;
    sal_Int32 mnEndIdx;
    sal_uInt32 mnFamily;
    ::std::vector< XMLPropertyState > &mrProperties;
    rtl::Reference < SvXMLImportPropertyMapper >   mxMapper;

public:

    SvXMLPropertySetContext(
            SvXMLImport& rImport, sal_Int32 nElement,
            const css::uno::Reference< css::xml::sax::XFastAttributeList >& xAttrList,
            sal_uInt32 nFamily,
            ::std::vector< XMLPropertyState > &rProps,
            rtl::Reference < SvXMLImportPropertyMapper > xMap,
              sal_Int32 nStartIdx = -1, sal_Int32 nEndIdx = -1 );
    SvXMLPropertySetContext(const SvXMLPropertySetContext&) = delete;
    SvXMLPropertySetContext(SvXMLPropertySetContext&&) = delete;
    void operator=(const SvXMLPropertySetContext&) = delete;
    void operator=(SvXMLPropertySetContext&&) = delete;

    virtual ~SvXMLPropertySetContext() override;

    virtual css::uno::Reference< css::xml::sax::XFastContextHandler > SAL_CALL createFastChildContext(
        sal_Int32 nElement,
        const css::uno::Reference< css::xml::sax::XFastAttributeList >& AttrList ) override;

    // This method is called from this instance implementation of
    // CreateChildContext if the element matches an entry in the
    // SvXMLImportItemMapper with the mid flag MID_FLAG_ELEMENT_ITEM_IMPORT
    virtual css::uno::Reference< css::xml::sax::XFastContextHandler > createFastChildContext(
        sal_Int32 nElement,
        const css::uno::Reference< css::xml::sax::XFastAttributeList >& xAttrList,
        ::std::vector< XMLPropertyState > &rProperties,
        const XMLPropertyState& rProp );

};

#endif // INCLUDED_XMLOFF_XMLPRCON_HXX

/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
