/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */
#ifndef INCLUDED_REPORTDESIGN_SOURCE_FILTER_XML_XMLCELL_HXX
#define INCLUDED_REPORTDESIGN_SOURCE_FILTER_XML_XMLCELL_HXX

#include <xmloff/xmlictxt.hxx>
#include <com/sun/star/report/XReportComponent.hpp>

namespace rptxml
{
    class ORptFilter;
    class OXMLTable;
    class OXMLCell : public SvXMLImportContext
    {
        css::uno::Reference< css::report::XReportComponent >  m_xComponent;
        OXMLTable*      m_pContainer;
        OXMLCell*       m_pCell;
        OUString        m_sStyleName;
        OUString        m_sText;
        sal_Int32       m_nCurrentCount;
        bool            m_bContainsShape;

        ORptFilter& GetOwnImport();
        OXMLCell(const OXMLCell&) = delete;
        OXMLCell& operator =(const OXMLCell&) = delete;
    public:

        OXMLCell( ORptFilter& rImport
                    ,const css::uno::Reference< css::xml::sax::XFastAttributeList > & xAttrList
                    ,OXMLTable* _pContainer
                    ,OXMLCell* _pCell = nullptr);
        virtual ~OXMLCell() override;

        virtual css::uno::Reference< css::xml::sax::XFastContextHandler > SAL_CALL createFastChildContext(
                sal_Int32 nElement, const css::uno::Reference< css::xml::sax::XFastAttributeList >& xAttrList ) override;

        virtual void SAL_CALL characters( const OUString& rChars ) override;
        virtual void SAL_CALL endFastElement(sal_Int32 nElement) override;

        void setComponent(const css::uno::Reference< css::report::XReportComponent >& _xComponent);
        void setContainsShape(bool _bContainsShapes);
    };

} // namespace rptxml


#endif // INCLUDED_REPORTDESIGN_SOURCE_FILTER_XML_XMLCELL_HXX

/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
